/***************************************************************************
 *cr                                                                       
 *cr            (C) Copyright 1995 The Board of Trustees of the           
 *cr                        University of Illinois                       
 *cr                         All Rights Reserved                        
 *cr                                                                   
 ***************************************************************************/

/***************************************************************************
 * RCS INFORMATION:
 *
 *	$RCSfile: POVDisplayDevice.C,v $
 *	$Author: dalke $	$Locker:  $		$State: Exp $
 *	$Revision: 1.7 $	$Date: 96/03/24 07:07:20 $
 *
 ***************************************************************************
 * DESCRIPTION:
 *
 * FileRenderer type for the Persistence of Vision raytracer
 *
 ***************************************************************************/
#include <string.h>
#include <stdio.h>
#include <math.h>
#include "POVDisplayDevice.h"
#include "Stack.h"
#include "Matrix4.h"
#include "DispCmds.h"
#include "Inform.h"
#include "utilities.h"

#define DEFAULT_RADIUS 0.002
#define DASH_LENGTH 0.02
#define PHONG_VALUE 0.8

///////////////////////// constructor and destructor

// constructor ... initialize some variables
POVDisplayDevice::POVDisplayDevice(char *nm) : FileRenderer(nm) {
	
    MSGDEBUG(1,"Creating POVDisplayDevice ..." << sendmsg);
    
    strcpy(renderCommand, "povray +H250 +W200 -I%s -O%s.tga +P +X +A +FT");
    strcpy(fname,"plot.pov");
}
        
// destructor
POVDisplayDevice::~POVDisplayDevice(void) { }

///////////////////////// protected nonvirtual routines

// draw a point
void POVDisplayDevice::point(float spdata[]) {
    float vec[3];

    // transform the world coordinates
    (transMat.top()).multpoint3d(spdata, vec);
   
    // draw the sphere
  
    fprintf(pov_file, "sphere {\n");  // sphere
    // center of sphere
    fprintf(pov_file, "  <%5f, %5f, %5f>, \n ", vec[0], vec[1], -vec[2]);
    // the radius of the sphere
    fprintf(pov_file, "  %5f \n", float(lineWidth)*DEFAULT_RADIUS ); 
    fprintf(pov_file, "  pigment { rgb<%3.2f, %3.2f, %3.2f> } \n", 
        currentColor[0], currentColor[1], currentColor[2]);
    fprintf(pov_file, "  finish{ phong %3.2f }\n", PHONG_VALUE);
    fprintf(pov_file, "}\n");
}

// draw a sphere
void POVDisplayDevice::sphere(float spdata[]) {
    float vec[3];
    float radius;
    
    // transform the world coordinates
    (transMat.top()).multpoint3d(spdata, vec);
    radius = scale_radius(spdata[3]);
   
    // draw the sphere
  
    fprintf(pov_file, "sphere {\n");  // sphere
    // center of sphere
    fprintf(pov_file, "  <%5f, %5f, %5f>, \n ", vec[0], vec[1], -vec[2]);
    // the radius of the sphere
    fprintf(pov_file, "  %5f \n", radius ); 
    fprintf(pov_file, "  pigment { rgb<%3.2f, %3.2f, %3.2f> } \n", 
        currentColor[0], currentColor[1], currentColor[2]);
    fprintf(pov_file, "  finish{ phong %3.2f }\n", PHONG_VALUE);
    fprintf(pov_file, "}\n");
}

// draw a line (cylinder) from a to b
void POVDisplayDevice::line(float *a, float*b) {
    int i, j, test;
    float dirvec[3], unitdirvec[3];
    float from[3], to[3], tmp1[3], tmp2[3];
    float len;
    
    if(lineStyle == ::SOLIDLINE ) {
  
        // transform the world coordinates
        (transMat.top()).multpoint3d(a, from);
        (transMat.top()).multpoint3d(b, to);
    
        // draw the cylinder
        fprintf(pov_file, "cylinder {\n"); // flat-ended cylinder
        // first point
        fprintf(pov_file, "  <%5f, %5f, %5f>,\n", from[0], from[1], -from[2]); 
        // second point
        fprintf(pov_file, "  <%5f, %5f, %5f>,\n ", to[0], to[1], -to[2]);
        // radius 2
        fprintf(pov_file, "  %5f \n", float(lineWidth)*DEFAULT_RADIUS);
        fprintf(pov_file, "  pigment { rgb<%3.2f, %3.2f, %3.2f> } \n", 
            currentColor[0], currentColor[1], currentColor[2]);
        fprintf(pov_file, "  finish{ phong %3.2f }\n", PHONG_VALUE);
        fprintf(pov_file, "}\n");
        
    } else if (lineStyle == ::DASHEDLINE ) {
        
         // transform the world coordinates
        (transMat.top()).multpoint3d(a, tmp1);
        (transMat.top()).multpoint3d(b, tmp2);

        // how to create a dashed line
        for(i=0;i<3;i++) {
            dirvec[i] = tmp2[i] - tmp1[i];  // vector from a to b
        }
        len = sqrtf( dirvec[0]*dirvec[0] + dirvec[1]*dirvec[1] + dirvec[2]*dirvec[2] );
        for(i=0;i<3;i++) {
            unitdirvec[i] = dirvec[i] / sqrtf(len);  // unit vector pointing from a to b
        }
           
        test = 1;
        i = 0;

        while( test == 1 ) {
            for(j=0;j<3;j++) {
                from[j] = tmp1[j] + (2*i)*DASH_LENGTH*unitdirvec[j];
                to[j] = tmp1[j] + (2*i + 1)*DASH_LENGTH*unitdirvec[j];
            }
            if( fabsf(tmp1[0] - to[0]) >= fabsf(dirvec[0]) ) {
                for(j=0;j<3;j++) {
                    to[j] = tmp2[j];
                }
                test = 0;
            }
    
            // draw the cylinder
            fprintf(pov_file, "cylinder {\n"); // flat-ended cylinder
            // first point
            fprintf(pov_file, "  <%5f, %5f, %5f>,\n", from[0], from[1], -from[2]); 
            // second point
            fprintf(pov_file, "  <%5f, %5f, %5f>,\n", to[0], to[1], -to[2]);
            // radius 2
            fprintf(pov_file, "  %5f \n", float(lineWidth)*DEFAULT_RADIUS);
            fprintf(pov_file, "  pigment { rgb<%3.2f, %3.2f, %3.2f> } \n", 
                currentColor[0], currentColor[1], currentColor[2]);
            fprintf(pov_file, "  finish{ phong %3.2f }\n", PHONG_VALUE);
            fprintf(pov_file, "}\n");
            
            i++;
        }
    } else {
        msgErr << "POVDisplayDevice: Unknown line style " << lineStyle << sendmsg;
    }

}

// draw a cylinder
void POVDisplayDevice::cylinder(float *a, float *b, float r) {
  float from[3], to[3];
  float radius;
  
  // transform the world coordinates
  (transMat.top()).multpoint3d(a, from);
  (transMat.top()).multpoint3d(b, to);
  radius = scale_radius(r);
    
  // draw the cylinder
  
        // draw the cylinder
        fprintf(pov_file, "cylinder {\n"); // flat-ended cylinder
        // first point
        fprintf(pov_file, "  <%5f, %5f, %5f>,\n", from[0], from[1], -from[2]); 
        // second point
        fprintf(pov_file, "  <%5f, %5f, %5f>,\n", to[0], to[1], -to[2]);
        // radius 2
        fprintf(pov_file, "  %5f \n", radius);
        fprintf(pov_file, "  pigment { rgb<%3.2f, %3.2f, %3.2f> } \n", 
            currentColor[0], currentColor[1], currentColor[2]);
        fprintf(pov_file, "  finish{ phong %3.2f }\n", PHONG_VALUE);
        fprintf(pov_file, "}\n");

}

// draw a cone
void POVDisplayDevice::cone(float *a, float *b, float r) {
  float from[3], to[3];
  float radius;
  
  // transform the world coordinates
  (transMat.top()).multpoint3d(a, from);
  (transMat.top()).multpoint3d(b, to);
  radius = scale_radius(r);
    
    
  // draw the cone
  
        // draw the cylinder
        fprintf(pov_file, "cone {\n"); // flat-ended cylinder
        // first point
        fprintf(pov_file, "  <%5f, %5f, %5f>,\n", from[0], from[1], -from[2]); 
        // radius 1
        fprintf(pov_file, "  %5f,\n", radius);
        // second point
        fprintf(pov_file, "  <%5f, %5f, %5f>,\n", to[0], to[1], -to[2]);
        // radius 2
        fprintf(pov_file, "  %5f \n", float(lineWidth)*DEFAULT_RADIUS);
        fprintf(pov_file, "  pigment { rgb<%3.2f, %3.2f, %3.2f> } \n", 
            currentColor[0], currentColor[1], currentColor[2]);
        fprintf(pov_file, "  finish{ phong %3.2f }\n", PHONG_VALUE);
        fprintf(pov_file, "}\n");
}

// draw a triangle
void POVDisplayDevice::triangle(float *a, float *b, float *c, float *n1, float *n2, float *n3) {
    float vec1[3], vec2[3], vec3[3];
    float norm1[3], norm2[3], norm3[3];
  
  
    // transform the world coordinates
    (transMat.top()).multpoint3d(a, vec1);
    (transMat.top()).multpoint3d(b, vec2);
    (transMat.top()).multpoint3d(c, vec3);
    (transMat.top()).multpoint3d(n1, norm1);
    (transMat.top()).multpoint3d(n2, norm2);
    (transMat.top()).multpoint3d(n3, norm3);

    // draw the triangle
  
    fprintf(pov_file, "smooth_triangle {\n"); // triangle
    // point one
    fprintf(pov_file, "  <%5f, %5f, %5f>,\n", vec1[0], vec1[1], -vec1[2]);
    // normal one
    fprintf(pov_file, "  <%5f, %5f, %5f>,\n", norm1[0], norm1[1], -norm1[2]);
    // point two
    fprintf(pov_file, "  <%5f, %5f, %5f>,\n", vec2[0], vec2[1], -vec2[2]); 
    // normal two
    fprintf(pov_file, "  <%5f, %5f, %5f>,\n", norm2[0], norm2[1], -norm2[2]); 
    // point three 
    fprintf(pov_file, "  <%5f, %5f, %5f>,\n", vec3[0], vec3[1], -vec3[2]);
    // normal three 
    fprintf(pov_file, "  <%5f, %5f, %5f>\n", norm3[0], norm3[1], -norm3[2]);
    fprintf(pov_file, "  pigment { rgb<%3.2f, %3.2f, %3.2f> } \n", 
        currentColor[0], currentColor[1], currentColor[2]);
    fprintf(pov_file, "  finish{ phong %3.2f }\n", PHONG_VALUE);
    fprintf(pov_file, "}\n");
  
}

// draw a square
void POVDisplayDevice::square(float *norm, float *a, float *b, float *c, float *d) {
  
     // draw as two triangles
     triangle(a, b, c, norm, norm, norm);
     triangle(a, c, d, norm, norm, norm);
  
}

///////////////////// public virtual routines

// initialize the file for output
int POVDisplayDevice::prepare3D(int bar) {

    bar = bar;

    if(!(pov_file = fopen(fname,"w"))) {
        msgErr << "Cannot open file " << fname << " for Raster3d output" << sendmsg;
        Initialized = FALSE;
    } else {
  
        Initialized = TRUE;
        pov_filename = stringdup(fname);
    
        // file for POV (persistance of vision) raytracer.
        
        fprintf(pov_file, "// POV input script : %s ", pov_filename);
        fprintf(pov_file, "for user %s\n", cuserid(NULL));
        fprintf(pov_file, "// try povray +H250 +W200 -I%s ", pov_filename);
        fprintf(pov_file, "-O%s.tga +P +X +A +FT\n", pov_filename);
        
        // Standard POV includes
        fprintf(pov_file, "#include \"colors.inc\"\n");
        fprintf(pov_file, "#include \"shapes.inc\"\n");
        fprintf(pov_file, "#include \"textures.inc\"\n");
                
        // Camera position
        // POV uses a left-handed coordinate system for some reason.
        // I like right-handed, so z(pov) = -z(vmd).
        
        fprintf(pov_file, "camera {\n");
        fprintf(pov_file, "  location  <%5f, %5f, %5f>\n", eyePos[0],
            eyePos[1], -eyePos[2]);
        fprintf(pov_file, "  look_at   <%5f, %5f, %5f>\n", eyeDir[0],
            eyeDir[1], -eyeDir[2]);
        fprintf(pov_file, "  up <0.0000, 1.0000, 0.0000>\n");
        fprintf(pov_file, "  right <%5f, 0.0000, 0.0000>\n", Aspect);
        fprintf(pov_file, "  direction <0, 0, 0.8>\n");
        fprintf(pov_file, "  sky <%5f, %5f, %5f>\n", upDir[0], upDir[1],
             -upDir[2]);
        fprintf(pov_file, "}\n");
        
        // Lights
        
        for(int i=0;i<DISP_LIGHTS;i++) {
            if(lightDefined[i] && lightOn[i]) {
                fprintf(pov_file, "light_source {\n");
                fprintf(pov_file, "  <%5f, %5f, %5f>\n", 
                    lightPos[i][0], lightPos[i][1], -lightPos[i][2]);
                fprintf(pov_file, "  color rgb<%3.2f, %3.2f, %3.2f>\n",
                    lightColor[i][0], lightColor[i][1], lightColor[i][2]);
                fprintf(pov_file, "}\n");
            }
       }
       
       // background color
       fprintf(pov_file, "background {\n");
       fprintf(pov_file, "  color rgb<%3.2f, %3.2f, %3.2f>\n", backColor[0], 
           backColor[1], backColor[2]);
       fprintf(pov_file, "}\n"); 
       
    }
    // that's all for the header.
    return Initialized;
}

    
// clean up after yourself
void POVDisplayDevice::update(int foo) {

  foo = foo;
  
  if(Initialized) {
    fclose(pov_file);
    if(pov_filename) delete [] pov_filename;
    msgInfo << "POV file generation finished" << sendmsg;
  }
}
